define("de_epages/shop/inc/topbar-controller", [
    "jquery",
    "ep"
], function($, ep) {

    var controllers = {},

        /*
         * Returns a name controller with the methods get, add, remove, show, hide and destroy.
         * Can be called with: controller['name'].method(arg)
         */
        topBarController = function(options) {
            var registeredApps = [],

                topBarElement;

            // add topbar element
            if (options && options.topBarElement && ep.isJQueryElement(options.topBarElement)) {
                topBarElement = options.topBarElement;
            }

            // via events
            // add
            $(document).on('addToTopBar.' + options.id, function(evt, app) {
                controllers[options.id].add(app);
            });
            // remove
            $(document).on('removeFromTopBar.' + options.id, function(evt, app) {
                controllers[options.id].remove(app);
            });

            // define and publish topbar module
            define('$pub!__topbar__', [], function() {
                // returns the controller object
                return controllers;
            });

            return {
                // get registered apps
                get: function() {
                    return registeredApps;
                },

                // add an app to the topbar controller
                add: function(app) {
                    if (!app.name) return false;

                    // add only if no app with the given name is already registered
                    if (registeredApps.indexOf(app.name) < 0) {
                        registeredApps.push(app.name);
                    }

                    return true;
                },

                // remove app from the topbar controller
                remove: function(app) {
                    if (!app.name) return false;

                    var index = registeredApps.indexOf(app.name);

                    if (index > -1) {
                        registeredApps.splice(index, 1);
                    }

                    if (registeredApps.length === 0) {
                        controllers[options.id].hide();
                    }

                    return index;
                },

                // hide topbar element
                hide: function() {
                    topBarElement && topBarElement.hide();
                },

                // show topbar element
                show: function() {
                    topBarElement && topBarElement.show();
                },

                destroy: function() {
                    // via events
                    // add
                    $(document).off('addToTopBar.' + options.id);
                    // remove
                    $(document).off('removeFromTopBar.' + options.id);
                }
            };
        };


    return function(options) {
        if (options === undefined) return false;

        // add new controller object to global controller object
        controllers[options.id ? options.id : 'default'] = topBarController(options);
    };
});