/*global define*/
/*jslint vars: true, plusplus: true, browser:true */
define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/presentation/ajaxtransport-test.html',
    'jquery',
    'de_epages/presentation/ajaxtransport',
    'testsuite/sinon',
    'testsuite/qunit-helper',

    'util/browser',
    '$ready!'
], function (QUnit, fixture, $, de_epages, sinon, helper) {
    'use strict';
    QUnit.module("de_epages/presentation/ajaxtransport", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("de_epages/presentation/ajaxtransport", function () {

        // define responses (sandbox.server.responses)
        var sandbox = sinon.sandbox.create(),
            server = sandbox.useFakeServer();

        server.autoRespond = true;
        server.autoRespondAfter = 0;
        server.respondWith("POST", "1?ErrorAction=JSONAjaxTransportFileError", [404, {}, '']);
        server.respondWith("POST", "0?ErrorAction=JSONAjaxTransportFileError", [200, {
            "Content-Type": "application/json"
        }, '{"upload":"Ok"}']);

        // create faked file inputs
        var inputs = [helper.fake('HTMLInputElement', {
            type: 'file',
            size: 20,
            files: [helper.fake('File', 'example-0.html', 'text/html', 512), helper.fake('File', 'example-0.jpg', 'image/jpeg', 2048)]
        }), helper.fake('HTMLInputElement', {
            type: 'file',
            size: 20,
            files: [helper.fake('File', 'example-1.jpg', 'image/jpeg', 2048)]
        }), helper.fake('HTMLInputElement', {
            type: 'file',
            size: 20,
            files: [helper.fake('File', 'example-2.jpg', 'image/jpeg', 40960)]
        }), helper.fake('HTMLInputElement', {
            type: 'file',
            size: 20,
            files: [helper.fake('File', 'example-3.json', 'application/json', 256)]
        }), helper.fake('HTMLInputElement', {
            type: 'file',
            size: 20,
            files: [helper.fake('File', 'example-4.jpg', 'image/jpeg', 2048)]
        })];

        var i = 0;

        // test vars
        var t = [{}, {}, {}, {}, {}];

        $(':file')
            .change(function (event) {
                var j = i;
                var ajax = $.ajax({
                    file: inputs[j],
                    url: j.toString(),
                    fileSizeMax: 4096,
                    fileMimeAccept: 'text/*,image/*',
                    dataType: 'json',
                    progress: function (event, statusText, jqXHR) {
                        //  console.log( event, statusText, jqXHR, 'DATA:', event.data );
                        t[j].progress = $.extend({}, event.data);
                    }
                })
                    .done(function (data, statusText, jqXHR) {
                        //  console.log( j, data, statusText, jqXHR );
                        t[j].success = $.merge([], data);
                    })
                    .fail(function (jqXHR, statusText, errorThrown) {
                        //  console.log( j, jqXHR, statusText, errorThrown );
                        t[j].error = errorThrown;
                    });

                if (i >= 4) {
                    ajax.abort();
                }

                i++;
            })
            .trigger('change')
            .trigger('change')
            .trigger('change')
            .trigger('change')
            .trigger('change');


        setTimeout(function () {
            var k = 0;

            if (!$.browser.msie) {
                QUnit.deepEqual(t[k].progress.item, {
                    "name": "NewFile",
                    "fileName": "example-0.jpg",
                    "fileSize": 2048,
                    "fileType": "image/jpeg"
                }, "$.ajax( case[" + k + "] ).progress()");
            }

            if (!$.browser.msie) {
                QUnit.deepEqual(t[k].success, [{
                    "upload": "Ok"
                }, {
                    "upload": "Ok"
                }], "$.ajax( case[" + k + "] ).success()");
            }

            if (!$.browser.msie) {
                QUnit.equal(
                    t[k].error,
                    undefined,
                    "$.ajax( case[" + k + "] ).fail()"
                );
            }

            k++;

            if (!$.browser.msie) {
                QUnit.deepEqual(t[k].progress.item, {
                    "name": "NewFile",
                    "fileName": "example-1.jpg",
                    "fileSize": 2048,
                    "fileType": "image/jpeg"
                }, "$.ajax( case[" + k + "] ).progress()");
            }

            QUnit.equal(
                t[k].success,
                undefined,
                "$.ajax( case[" + k + "] ).success()"
            );

            if (!$.browser.msie) {
                QUnit.equal(
                    t[k].error,
                    'Items of Request Failed',
                    "$.ajax( case[" + k + "] ).fail()"
                );
            }


            k++;

            if (!$.browser.msie) {
                QUnit.deepEqual(t[k].progress.item, {
                    "name": "NewFile",
                    "fileName": "example-2.jpg",
                    "fileSize": 40960,
                    "fileType": "image/jpeg"
                }, "$.ajax( case[" + k + "] ).progress()");
            }


            QUnit.equal(
                t[k].success,
                undefined,
                "$.ajax( case[" + k + "] ).success()"
            );


            if (!$.browser.msie) {
                QUnit.equal(
                    t[k].error,
                    'Items of Request Failed',
                    "$.ajax( case[" + k + "] ).fail()"
                );
            }


            k++;

            if (!$.browser.msie) {
                QUnit.deepEqual(t[k].progress.item, {
                    "name": "NewFile",
                    "fileName": "example-3.json",
                    "fileSize": 256,
                    "fileType": "application/json"
                }, "$.ajax( case[" + k + "] ).progress()");
            }

            QUnit.equal(
                t[k].success,
                undefined,
                "$.ajax( case[" + k + "] ).success()"
            );

            if (!$.browser.msie) {
                QUnit.equal(
                    t[k].error,
                    'Items of Request Failed',
                    "$.ajax( case[" + k + "] ).fail()"
                );
            }

            k++;

            if (!$.browser.msie) {
                QUnit.equal(
                    t[k].progress,
                    undefined,
                    "$.ajax( case[" + k + "] ).progress()"
                );
            }

            QUnit.equal(
                t[k].success,
                undefined,
                "$.ajax( case[" + k + "] ).success()"
            );

            if (!$.browser.msie) {
                QUnit.equal(
                    t[k].error,
                    'abort',
                    "$.ajax( case[" + k + "] ).abort()"
                );
            }

            QUnit.start();

        }, 500);
    });
});
