/*globals define*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!ep/ui/videodialog-test',
    'jquery',
    'ep/ui/videodialog',

    '$ready!'
], function (QUnit, fixture, $, video) {

    QUnit.module("ep/ui/videodialog", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.test('ep/ui/videodialog - check existence of videodialog methods', function () {
        $.each(['_create', '_youtube', 'open', 'close', 'destroy'], function () {
            QUnit.ok(
                $.isFunction($.ui.uiVideodialog.prototype[this]),
                this + ' is a method of the videodialog widget'
            );
        });
    });

    QUnit.asyncTest("ep/ui/videodialog - UI tests for youtube video", function () {
        var $img = $('.ep-helpcenter-video-list-image');

        // instantiate videodialog widget
        $img.uiVideodialog({
          type: {
            name: 'youtube',
            url: $img.data('videourl'),
            id: $img.data('videoid'),
            ext: $img.data('videoext')
          }
        });

        // wait until widget is instantiated
        window.setTimeout(function() {
            QUnit.equal(
                !!$img.uiVideodialog('instance'),
                true,
                'uiVideodialog widget instance exists.'
            );

            // trigger click to open dialog
            $img.trigger('click');
            // wait until dialog is open
            window.setTimeout(function() {
                QUnit.equal(
                    $img.data('uiUiVideodialog').dialog.length,
                    1,
                    'Dialog was created.'
                );
                QUnit.equal(
                    $('.ui-dialog').length,
                    1,
                    'Dialog element is present in DOM.'
                );
                QUnit.equal(
                    $img.data('uiUiVideodialog').dialog.is(':visible'),
                    true,
                    'Dialog is visible.'
                );

                // use public method to close dialog
                $img.uiVideodialog('close');
                // wait until dialog is closed
                window.setTimeout(function() {
                    QUnit.equal(
                        $img.data('uiUiVideodialog').dialog.is(':visible'),
                        false,
                        'Dialog is closed.'
                    );

                    // destroy widget
                    $img.uiVideodialog('destroy');
                    // wait until widget is destroyed
                    window.setTimeout(function() {
                        QUnit.equal(
                            $img.data('uiUiVideodialog') === undefined,
                            true,
                            'Widget was destroyed.'
                        );
                        QUnit.equal(
                            $('.ui-dialog').length,
                            0,
                            'Dialog element was removed from DOM.'
                        );

                        QUnit.start();
                    }, 500);
                }, 500);
            }, 500);
        }, 500);
    });

    QUnit.asyncTest("ep/ui/videodialog - UI tests for vimeo video", function () {
        var $img = $('.ep-helpcenter-video-list-image');

        // instantiate videodialog widget
        $img.uiVideodialog({
          type: {
            name: 'vimeo',
            id: 312111187,
            video: {"width":640,"author_name":"ePages Marketing","provider_url":"https://vimeo.com/","thumbnail_width":640,"html":"<iframe class=\"ep-video-iframe\" src=\"//?ViewAction=DereferredRedirect&URL=https%3A//player.vimeo.com/video/312111187%3Fapp_id%3D122963%26dnt%3D1&nonce=NWY3ZTE2MzM5ODEzZWNkY2MwNDQ1MWQ1YzkxYWI5NmVhOTllYjFmNThmMjk2NDdmZDlkNzcwYjc2Mzg4MDVmYl8xNTUxNjkwNDM5\" width=\"640\" height=\"360\" frameborder=\"0\" title=\"Das Design Ihres Onlineshops\" allow=\"autoplay; fullscreen\" allowfullscreen></iframe>","video_id":312111187,"author_url":"https://vimeo.com/user94000187","version":"1.0","provider_name":"Vimeo","is_plus":"0","duration":148,"thumbnail_url":"https://i.vimeocdn.com/video/753086085.jpg","height":360,"description":"","uri":"/videos/312111187","thumbnail_height":360,"thumbnail_url_with_play_button":"https://i.vimeocdn.com/filter/overlay?src0=https%3A%2F%2Fi.vimeocdn.com%2Fvideo%2F753086085_640.jpg&src1=http%3A%2F%2Ff.vimeocdn.com%2Fp%2Fimages%2Fcrawler_play.png","title":"Das Design Ihres Onlineshops","type":"video","upload_date":"2019-01-18 09:33:40","account_type":"pro"}
          }
        });

        // wait until widget is instantiated
        window.setTimeout(function() {
            QUnit.equal(
                !!$img.uiVideodialog('instance'),
                true,
                'uiVideodialog widget instance exists.'
            );


            // trigger click to open dialog
            $img.trigger('click');
            // wait until dialog is open
            window.setTimeout(function() {
                QUnit.equal(
                    $img.data('uiUiVideodialog').dialog.length,
                    1,
                    'Dialog was created.'
                );

                QUnit.equal(
                    $('.ui-dialog').length,
                    1,
                    'Dialog element is present in DOM.'
                );
                QUnit.equal(
                    $img.data('uiUiVideodialog').dialog.is(':visible'),
                    true,
                    'Dialog is visible.'
                );

                // use public method to close dialog
                $img.uiVideodialog('close');
                // wait until dialog is closed
                window.setTimeout(function() {
                    QUnit.equal(
                        $img.data('uiUiVideodialog').dialog.is(':visible'),
                        false,
                        'Dialog is closed.'
                    );

                    // destroy widget
                    $img.uiVideodialog('destroy');
                    // wait until widget is destroyed
                    window.setTimeout(function() {
                        QUnit.equal(
                            $img.data('uiUiVideodialog') === undefined,
                            true,
                            'Widget was destroyed.'
                        );
                        QUnit.equal(
                            $('.ui-dialog').length,
                            0,
                            'Dialog element was removed from DOM.'
                        );

                        QUnit.start();
                    }, 500);
                }, 500);
            }, 500);
        }, 500);
    });
});